/*
 * Copyright© Econoinfo Tecnologia da Informação LTDA.
 * Todos os direitos reservados (All rights reserved)
 * 
 * O conteúdo deste documento, no todo ou em parte, é um bem de propriedade intelectual da EconoInfo®. É, portanto, proibida a utilização, publicação
 * ou reprodução não autorizada, sob pena de violação dos direitos da EconoInfo®.
 */
package app;


import java.lang.management.ManagementFactory;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.ejb.ConcurrencyManagement;
import javax.ejb.ConcurrencyManagementType;
import javax.ejb.EJB;
import javax.ejb.Singleton;
import javax.ejb.Startup;
import javax.management.InstanceAlreadyExistsException;
import javax.management.MBeanRegistrationException;
import javax.management.MBeanServer;
import javax.management.MalformedObjectNameException;
import javax.management.NotCompliantMBeanException;
import javax.management.ObjectName;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * @author wallace
 * 
 */
/**
 * Class JMXServiceActivator is a Singleton Session Bean which accepts launch methods using JMX technology.
 * 
 * @author orair
 * 
 */
@Singleton
@ConcurrencyManagement(ConcurrencyManagementType.BEAN)
// Garantee it is available any time to be lauched using JMX invocations
@Startup
public class JMXServiceActivator implements JMXServiceActivatorMXBean {

	private static Logger logger = LoggerFactory.getLogger(JMXServiceActivator.class);

	@EJB
	private GetRepositoryEjbImpl ejbImpl;

	@Override
	public void getRepositoryUsingExplicitJNDI(String jndiName) {
		ejbImpl.getRepositoryUsingExplicitJNDI(jndiName);
	}

	@Override
	public void getRepositoryUsingJCRUtils(String jndiName) {
		ejbImpl.getRepositoryUsingJCRUtils(jndiName);
	}

	@Override
	public void getRepositoryUsingExplicitJNDIWithoutProxy(String jndiName) {
		// Doesn't work in any way.
		App.getRepositoryUsingExplicitJNDI(jndiName);
	}

	@Override
	public void getRepositoryUsingJCRUtilsWithoutProxy(String jndiName) {
		App.getRepositoryUsingJCRUtils(jndiName);
	}

	@PostConstruct
	public void registerInManagedBeanServer() {
		// Get the Platform MBean Server
		MBeanServer mbs = ManagementFactory.getPlatformMBeanServer();

		try {
			// Construct the ObjectName for the MBean we will register
			ObjectName name = new ObjectName(getRegisterName());
			if (mbs.isRegistered(name) == false) {
				// Register the MBean
				mbs.registerMBean(this, name);
			}
		} catch (MalformedObjectNameException e) {
			logger.warn("Nome da classe managed bean mal formada.", e);
		} catch (InstanceAlreadyExistsException e) {
			logger.warn("Classe managed bean já existente.", e);
		} catch (MBeanRegistrationException e) {
			logger.warn("Problemas no registro da classe managed bean.", e);
		} catch (NotCompliantMBeanException e) {
			logger.warn("Managed Bean a ser registrado não está de acordo com a interface esperada.", e);
		}
	}

	/**
	 * Remove register from managed bean server.
	 * 
	 * <p>
	 * 
	 * @PreDestroy annotation is needed to certify that this Singleton will be unregistered in managed bean server when undeployed.<br>
	 *             It avoids memory leaks.
	 *             </p>
	 */
	@PreDestroy
	public void removeRegistroNoManagedBeanServer() {
		// Get the Platform MBean Server
		MBeanServer mbs = ManagementFactory.getPlatformMBeanServer();

		try {
			// Construct the ObjectName for the MBean
			ObjectName name = new ObjectName(getRegisterName());
			if (mbs.isRegistered(name)) {
				mbs.unregisterMBean(name);
			} else {
				logger.warn("Esperava-se que houvesse o registro de <" + name + "> no managed bean server, porém este registro não foi encontrado.");
			}
		} catch (Exception e) {
			throw new IllegalStateException("Problem during unregistration of Monitoring into JMX:" + e);
		}
	}

	/**
	 * @return register namee
	 */
	private String getRegisterName() {
		return (this.getClass().getPackage().getName() + ":type=" + this.getClass().getSimpleName());
	}
}
